/* rbcrack.c - a Ruby interface to CrackLib
 * 
 * $Id: rbcrack.c,v 1.10 2003/06/11 06:43:58 ianmacd Exp $
 *
 * Version : 0.4.0
 * Author  : Ian Macdonald <ian@caliban.org>
 *
 * Copyright (C) 2002 Ian Macdonald
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <ruby.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#ifdef HAVE_PACKER_H
#include <packer.h>
#else
#include <crack.h>
#endif

#include "rbcrack.h"

#define CRACK_VERSION "0.4.0"


VALUE ePassword_BadDictionary;
VALUE ePassword_WeakPassword;

/* Password.initialize */
static VALUE passwd_init(VALUE self, VALUE passwd)
{

  /* call superclass's initialize method */
  rb_call_super(1, &passwd);

  return passwd;
}

/* Password.check */
static VALUE passwd_check(VALUE self, VALUE args)
{
  VALUE dict;
  char *objection;
  char *buffer;

  /* pop the one and only argument we may have been passed */
  dict = rb_ary_pop(args);

  if (dict == Qnil || strcmp(STR2CSTR(dict), "") == 0) {
    /* no argument passed, so use default location from rbcrack.h */
    dict = rb_str_new2(CRACK_DICT);
  } else {
    buffer = malloc(strlen(STR2CSTR(dict)) + 8);
    strcpy(buffer, STR2CSTR(dict));
    strcat(buffer, ".pwd");

    if (access(buffer, R_OK) != 0) {
      free(buffer);
      rb_raise(ePassword_BadDictionary, "%s", strerror(errno));
    }

    free(buffer);

  }
    
  /* perform check on password */
  objection = FascistCheck(STR2CSTR(self), STR2CSTR(dict));

  /* return true on success; raise an exception otherwise */
  if (objection) {
    rb_raise(ePassword_WeakPassword, "%s", objection);
  } else {
    return Qtrue;
  }

}

/* initialize this class */
void Init_crack()
{
  VALUE cPassword;

  /* define the Password class */
  cPassword = rb_define_class("Password", rb_cString);

  /* define the Password::BadDictionary exception */
  ePassword_BadDictionary = rb_define_class_under(cPassword, "BadDictionary",
						  rb_eRuntimeError);

  /* define the Password::WeakPassword exception */
  ePassword_WeakPassword = rb_define_class_under(cPassword, "WeakPassword",
						 rb_eRuntimeError);

  /* define a version constant */
  rb_define_const(cPassword, "VERSION", rb_str_new2(CRACK_VERSION));

  /* define the Password.initialize method */
  rb_define_method(cPassword, "initialize", passwd_init, 1);

  /* enable the same method in the superclass */
  rb_enable_super(cPassword, "initialize");

  /* define the Password.check method */
  rb_define_method(cPassword, "check", passwd_check, -2);

  return;
}
